package ij.plugin;
import ij.*;
import ij.gui.*;
import ij.process.*;

/** This plug-in animates stacks. */
public class Animator implements PlugIn {

	private static double animationSpeed = 10.0; // frames per second

	/** Set 'arg' to "set" to display a dialog that allows the user to specify the
		animation speed. Set it to "start" to start animating the current stack.
		Set it to "stop" to stop animation. Set it to "next" or "previous"
		to stop any animation and display the next or previous frame. 
	*/
	public void run(String arg) {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null)
			{IJ.noImage(); return;}
    	int nSlices = imp.getStackSize();
		if (nSlices<2)
			{IJ.error("Stack required."); return;}
		ImageWindow win = imp.getWindow();
		if (!(win instanceof StackWindow))
			return;
		StackWindow swin = (StackWindow)win;
		ImageStack stack = imp.getStack();
		int slice = imp.getCurrentSlice();
		IJ.register(Animator.class);
		
		if (arg.equals("set")) {
			double speed = IJ.getNumber("Animation Speed (1-100fps):", animationSpeed);
			if (speed==IJ.CANCELED) return;
			if (speed>100.0) speed = 100.0;
			if (speed<1.0) speed = 1.0;
			animationSpeed = speed;
			return;
		}
			
		if (arg.equals("start")) {
			if (!imp.lock()) return;
			swin.running = true;
			long time, nextTime=System.currentTimeMillis();
			Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
			while (swin.running) {
				time = System.currentTimeMillis();
				if (time<nextTime)
					IJ.wait((int)(nextTime-time));
				else
					Thread.yield();
				nextTime += (long)(1000.0/animationSpeed);
				slice++;
				if (slice>nSlices)
					slice = 1;
				swin.showSlice(slice);
			}
			imp.unlock();
			return;
		}

		if (swin.running) { // "stop", "next" and "previous" all stop animation
			// stop animating
			swin.running = false;
			IJ.wait(500+(int)(1000.0/animationSpeed));
			imp.unlock(); 
		}

		if (arg.equals("stop")) {
			return;
		}
			
		if (arg.equals("next")) {
			if (!imp.lock()) return;
			swin.showSlice(++slice);
			imp.unlock();
			return;
		}
		
		if (arg.equals("previous")) {
			if (!imp.lock()) return;
			swin.showSlice(--slice);
			imp.unlock();
			return;
		}
	}

}

