package ij.plugin.filter;
import java.awt.Color;
import ij.*;
import ij.gui.*;
import ij.process.*;
import ij.plugin.filter.ParticleAnalyzer;
import ij.measure.*;

/** Implements ImageJ's Analyze/Analyze Line Graph command. */
public class LineGraphAnalyzer implements PlugInFilter, Measurements  {
	ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_8G+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		analyze(imp);
	}
	
	/** Uses ImageJ's particle analyzer to extract a set
		of coordinate pairs from a digitized line graph. */
	public void analyze(ImagePlus imp) {
		ByteProcessor ip = (ByteProcessor)imp.getProcessor();
		int width = ip.getWidth();
		int height = ip.getHeight();
		ImageProcessor ip2 = new ByteProcessor(width, height, (byte[])ip.getPixelsCopy(), null);
		ip2.setColor(Color.white);
		for (int i=1; i<width; i+=2) {
			ip2.moveTo(i,0);
			ip2.lineTo(i,height-1);
		}
		ip2 = ip2.rotateRight();
		ImagePlus imp2 = imp.createImagePlus();
		imp2.setProcessor("Temp", ip2);
		Calibration cal = imp2.getCalibration();
		double pw = cal.pixelWidth;
		double ph = cal.pixelHeight;
		cal.pixelWidth = ph;
		cal.pixelHeight = pw;
		imp2.setCalibration(cal);
		if (IJ.altKeyDown()) imp2.show();
		
		int options = ParticleAnalyzer.SHOW_PROGRESS;
		int measurements = CENTROID;
		int minSize = 1;
		int maxSize = 999999;
		ParticleAnalyzer pa = new ParticleAnalyzer(options,measurements,minSize,maxSize);
		pa.analyze(imp2);
		ResultsTable rt = pa.getResultsTable();
		float[] y = rt.getColumn(1);				
		float[] x = rt.getColumn(2);
		String units = " ("+cal.getUnits()+")";
		String xLabel = "X"+units;
		String yLabel = "Y"+units;
		new PlotWindow("Line Graph", xLabel, yLabel, x, y).draw();				
	}
	
}
